<?php

declare(strict_types=1);

namespace Drupal\migrate_dc\Plugin\migrate\process;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\Attribute\MigrateProcess;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row as MigrateRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[MigrateProcess(
  id: 'migrate_dc_uri_transformator',
)]
class MigrateDcUriTransformator extends ProcessPluginBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ): static {
    // @phpstan-ignore-next-line
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritDoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritDoc}
   */
  public function transform(
    $value,
    MigrateExecutableInterface $migrate_executable,
    MigrateRow $row,
    $destination_property,
  ) {
    $uriParts = parse_url($value);
    if (isset($uriParts['scheme'])
      && $uriParts['scheme'] === 'entity.uuid'
      && !empty($uriParts['path'])
    ) {
      [$entityTypeId, $uuid] = explode('/', $uriParts['path']);
      $entityId = $this->getEntityIdFromUuid($entityTypeId, $uuid);
      if ($entityId) {
        $value = "entity:$entityTypeId/$entityId";
      }
    }

    return $value;
  }

  /**
   * @noinspection PhpUnhandledExceptionInspection
   */
  protected function getEntityIdFromUuid(string $entityTypeId, string $uuid): ?string {
    if (!$this->entityTypeManager->hasDefinition($entityTypeId)) {
      return NULL;
    }

    $entityType = $this->entityTypeManager->getDefinition($entityTypeId);
    if (!$entityType->hasKey('uuid')) {
      return NULL;
    }

    /** @var array<string> $result */
    $result = $this
      ->entityTypeManager
      ->getStorage($entityTypeId)
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition((string) $entityType->getKey('uuid'), $uuid)
      ->execute();

    return $result ? reset($result) : NULL;
  }

}
