<?php

declare(strict_types=1);

namespace Drupal\migrate_dc\Plugin\migrate_plus\data_parser;

use Drupal\Component\Serialization\SerializationInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\MigrateException;
use Drupal\migrate_plus\DataParserPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Obtain Yaml data for migration.
 *
 * @\Drupal\migrate_plus\Annotation\DataParser(
 *   id = "migrate_dc_yaml",
 *   title = @\Drupal\Core\Annotation\Translation("Yaml")
 * )
 */
class MigrateDcYamlParser extends DataParserPluginBase implements ContainerFactoryPluginInterface {

  /**
   * Iterator over the Yaml data.
   */
  protected \Iterator $iterator;

  protected SerializationInterface $serializer;

  /**
   * {@inheritdoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   * @phpstan-param array<string, mixed> $plugin_definition
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ): static {
    // @phpstan-ignore-next-line
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('serialization.yaml'),
    );
  }

  /**
   * {@inheritdoc}
   *
   * @phpstan-param array<string, mixed> $configuration
   * @phpstan-param string $plugin_id
   * @phpstan-param array<string, mixed> $plugin_definition
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    SerializationInterface $serializer,
  ) {
    $this->serializer = $serializer;

    if (empty($configuration['ids'])) {
      throw new MigrateException('You must declare "ids" as a unique array of fields in your source settings.');
    }

    $configuration += [
      'item_selector' => '',
    ];

    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * @return array<string, mixed>
   *
   * @throws \GuzzleHttp\Exception\RequestException
   */
  protected function getSourceData(string $url): array {
    $response = $this->getDataFetcherPlugin()->getResponseContent($url);
    $sourceData = $this->serializer->decode($response);

    if (is_int($this->itemSelector)) {
      return $this->selectByDepth($sourceData);
    }

    // Otherwise, we're using xpath-like selectors.
    $selectors = array_filter(explode('/', trim($this->itemSelector, '/')));
    foreach ($selectors as $selector) {
      $sourceData = $sourceData[$selector];
    }

    return $sourceData;
  }

  /**
   * Get the source data for reading.
   *
   * @param array<mixed> $rawData
   *
   * @return array<mixed>
   */
  protected function selectByDepth(array $rawData): array {
    // Return the results in a recursive iterator that can traverse
    // multidimensional arrays.
    $iterator = new \RecursiveIteratorIterator(
      new \RecursiveArrayIterator($rawData),
      \RecursiveIteratorIterator::SELF_FIRST
    );

    $items = [];
    // Backwards-compatibility - an integer item_selector is interpreted as a
    // depth.
    // When there is an array of items at the expected depth, pull that array
    // out as a distinct item.
    $identifierDepth = $this->itemSelector;
    $iterator->rewind();
    while ($iterator->valid()) {
      $item = $iterator->current();
      if (is_array($item) && $iterator->getDepth() == $identifierDepth) {
        $items[] = $item;
      }

      $iterator->next();
    }

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  protected function openSourceUrl($url): bool {
    // (Re)open the provided URL.
    $source_data = $this->getSourceData($url);
    $this->iterator = new \ArrayIterator($source_data);

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  protected function fetchNextRow(): void {
    $current = $this->iterator->current();
    if ($current) {
      foreach ($this->fieldSelectors() as $fieldName => $selector) {
        $field_data = $current;
        $fieldSelectors = explode('/', trim($selector, '/'));
        foreach ($fieldSelectors as $fieldSelector) {
          $field_data = $field_data[$fieldSelector];
        }
        $this->currentItem[$fieldName] = $field_data;
      }

      if (!empty($this->configuration['include_raw_data'])) {
        $this->currentItem['raw'] = $current;
      }

      $this->iterator->next();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function fieldSelectors(): array {
    $fields = [];
    foreach ($this->configuration['fields'] as $key => $fieldInfo) {
      if (!is_array($fieldInfo)) {
        $fields[$key] = $key;
      }
      elseif (isset($fieldInfo['selector'])) {
        $fields[$fieldInfo['name']] = $fieldInfo['selector'];
      }
    }

    return $fields;
  }

}
