<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Kernel\Plugin\migrate\process;

use Drupal\migrate\MigrateException;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\MigrateMessageInterface;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcFileContent;
use Drupal\KernelTests\KernelTestBase;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;

#[Group('migrate_dc')]
#[CoversClass(MigrateDcFileContent::class)]
class MigrateDcFileContentTest extends KernelTestBase implements MigrateMessageInterface {

  /**
   * {@inheritdoc}
   *
   * @var array<string>
   */
  protected static $modules = [
    'migrate_dc',
    'migrate',
    'file',
    'system',
    'user',
    'profile',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('file');
    $this->installEntitySchema('user');
    $this->installConfig(static::$modules);
  }

  /**
   * @param array<string, mixed> $configuration
   */
  protected function createProcessorInstance(array $configuration): MigrateDcFileContent {
    return new MigrateDcFileContent(
      $configuration,
      'migrate_dc_file_content',
      [],
    );
  }

  protected function createMigrateExecutable(): MigrateExecutableInterface {
    /** @var \Drupal\migrate\Plugin\MigrationPluginManagerInterface $migrationPluginManager */
    $migrationPluginManager = \Drupal::getContainer()->get('plugin.manager.migration');
    $migration = $migrationPluginManager->createStubMigration([]);

    return new MigrateExecutable(
      $migration,
      // @todo Create a Dummy messenger.
      $this,
    );
  }

  /**
   * @return array<string, mixed>
   */
  public static function casesTransformSuccess(): array {
    $fileContent = 'dummy text';
    $filePath = 'public://MigrateDcFileContentTest';
    $fileName = 'fileName.txt';

    return [
      'file exists' => [
        'expected' => $fileContent,
        'configuration' => [
          'path' => $filePath,
          'file_name' => $fileName,
        ],
        'destination' => [
          'property' => '',
          'value' => '',
        ],
        'fileToCreate' => [
          'uri' => "$filePath/$fileName",
          'content' => $fileContent,
        ],
      ],
      'file exists with @ in path' => [
        'expected' => $fileContent,
        'configuration' => [
          'path' => '@a/b/c',
          'file_name' => $fileName,
        ],
        'destination' => [
          'property' => 'a/b/c',
          'value' => $filePath,
        ],
        'fileToCreate' => [
          'uri' => "$filePath/$fileName",
          'content' => $fileContent,
        ],
      ],
      'file does not exist' => [
        'expected' => '',
        'configuration' => [
          'path' => 'foo',
          'file_name' => "bar",
        ],
        'destination' => [
          'property' => '',
          'value' => '',
        ],
      ],
    ];
  }

  /**
   * @phpstan-param array<string, mixed> $configuration
   * @phpstan-param array<string, mixed> $destination
   * @phpstan-param null|array<string, mixed> $fileToCreate
   */
  #[DataProvider('casesTransformSuccess')]
  public function testTransformSuccess(
    mixed $expected,
    array $configuration,
    array $destination,
    ?array $fileToCreate = NULL,
  ): void {
    // @todo Unique directory name.
    /** @var \Drupal\Core\File\FileSystemInterface $filesystem */
    $filesystem = \Drupal::getContainer()->get('file_system');
    $filesystem->mkdir('public://MigrateDcFileContentTest', 0777);

    if ($fileToCreate) {
      file_put_contents($fileToCreate['uri'], $fileToCreate['content']);
    }

    $value = '';
    $row = new MigrateRow();
    $row->setDestinationProperty($destination['property'], $destination['value']);

    $migrateExecutable = $this->createMigrateExecutable();
    $processor = $this->createProcessorInstance($configuration);

    /* @noinspection PhpUnhandledExceptionInspection */
    static::assertSame(
      $expected,
      $processor->transform(
        $value,
        $migrateExecutable,
        $row,
        '',
      ),
    );
  }

  /**
   * @return array<string, array<string, mixed>>
   */
  public static function casesTransformFail(): array {
    return [
      'no file in config' => [
        'expected' => MigrateException::class,
        'configuration' => [],
        'destination' => [
          'property' => '',
          'value' => '',
        ],
      ],
    ];
  }

  /**
   * @phpstan-param class-string<\Throwable> $expected
   * @phpstan-param array<string, mixed> $configuration
   * @phpstan-param array<string, mixed> $destination
   */
  #[DataProvider('casesTransformFail')]
  public function testTransformFail(string $expected, array $configuration, array $destination): void {
    $value = '';
    $row = new MigrateRow();
    $row->setDestinationProperty($destination['property'], $destination['value']);

    $migrateExecutable = $this->createMigrateExecutable();
    $processor = $this->createProcessorInstance($configuration);
    $this->expectException($expected);
    $this->expectExceptionMessage('Config path value does not exist');

    $processor->transform(
      $value,
      $migrateExecutable,
      $row,
      '',
    );
  }

  /**
   * {@inheritdoc}
   *
   * @todo Move this into a separated helper class.
   */
  public function display($message, $type = 'status'): void {
    static::assertTrue($type === 'status', $message);
  }

}
