<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Kernel\Plugin\migrate\process;

use Drupal\commerce_price\Price;
use Drupal\commerce_shipping\ShipmentItem;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\MigrateMessageInterface;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcShipmentItem;
use Drupal\physical\Weight;
use Drupal\physical\WeightUnit;
use Drupal\KernelTests\KernelTestBase;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;

/**
 * @noinspection PhpUnhandledExceptionInspection
 */
#[Group('migrate_dc')]
#[CoversClass(MigrateDcShipmentItem::class)]
class MigrateDcShipmentItemTest extends KernelTestBase implements MigrateMessageInterface {

  /**
   * {@inheritdoc}
   *
   * @var array<string>
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'options',
    'views',
    'path',
    'path_alias',
    'token',
    'text',
    'physical',
    'address',
    'entity_reference_revisions',
    'state_machine',
    'inline_entity_form',
    'profile',
    'commerce',
    'commerce_store',
    'commerce_number_pattern',
    'commerce_price',
    'commerce_product',
    'commerce_order',
    'commerce_shipping',
    'migrate',
    'migrate_dc',
  ];

  /**
   * {@inheritDoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('commerce_store');
    $this->installEntitySchema('commerce_product');
    $this->installEntitySchema('commerce_order');
    $this->installEntitySchema('commerce_shipment');
    $this->installConfig(static::$modules);
  }

  /**
   * @return array<string, mixed>
   */
  public static function casesTransform(): array {
    return [
      'basic' => [
        'expected' => [
          new ShipmentItem([
            'order_item_id' => '32',
            'title' => 'a',
            'quantity' => 1,
            'weight' => new Weight('2', WeightUnit::KILOGRAM),
            'declared_value' => new Price('10', 'usd'),
          ]),
        ],
        'value' => [
          [
            'order_item_id' => '32',
            'title' => 'a',
            'quantity' => '1',
            'weight' => [
              'number' => '2',
              'unit' => WeightUnit::KILOGRAM,
            ],
            'declared_value' => [
              'number' => '10',
              'currency_code' => 'USD',
            ],
          ],
        ],
      ],
    ];
  }

  /**
   * @phpstan-param array<\Drupal\commerce_shipping\ShipmentItem> $expected
   * @phpstan-param array<mixed> $value
   */
  #[DataProvider('casesTransform')]
  public function testTransform(array $expected, array $value): void {
    $pluginId = '';
    $pluginDefinition = [];
    $configuration = [];
    $destinationProperty = '';
    $row = new MigrateRow();

    $container = \Drupal::getContainer();
    /** @var \Drupal\migrate\Plugin\MigrationPluginManagerInterface $migrationPluginManager */
    $migrationPluginManager = $container->get('plugin.manager.migration');
    $migration = $migrationPluginManager->createStubMigration([]);
    $migrateExecutable = new MigrateExecutable($migration, $this);

    $migrateShipmentItem = new MigrateDcShipmentItem(
      $configuration,
      $pluginId,
      $pluginDefinition,
    );

    static::assertEquals(
      $expected,
      $migrateShipmentItem->transform(
        $value,
        $migrateExecutable,
        $row,
        $destinationProperty,
      ),
    );
  }

  /**
   * {@inheritDoc}
   */
  public function display($message, $type = 'status'): void {
    static::assertTrue($type == 'status', $message);
  }

}
