<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Kernel\Plugin\migrate\process;

use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\MigrateMessageInterface;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcStrToTime;
use Drupal\KernelTests\KernelTestBase;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;

#[Group('migrate_dc')]
#[CoversClass(MigrateDcStrToTime::class)]
class MigrateDcStrToTimeTest extends KernelTestBase implements MigrateMessageInterface {

  /**
   * {@inheritDoc}
   *
   * @var array<string>
   */
  protected static $modules = [
    'migrate',
    'migrate_plus',
    'migrate_dc',
  ];

  /**
   * {@inheritDoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(static::$modules);
  }

  /**
   * @return array<string, array<string, mixed>>
   */
  public static function casesTransform(): array {
    return [
      'basic' => [
        'expected' => [
          'value' => '1977-07-15T12:00:00',
          'requestTime' => strtotime('1977-07-12 12:00:00'),
          'valueToDateFormatter' => strtotime('1977-07-15 12:00:00'),
        ],
        'configuration' => [
          'output_format' => 'Y-m-d\TH:i:s',
          'base_time' => '1977-07-12 12:00:00',
        ],
        'value' => '+3 days',
      ],
      'unix' => [
        'expected' => [
          'value' => 42,
          'requestTime' => strtotime('1977-07-12 12:00:00'),
          'valueToDateFormatter' => 42,
        ],
        'configuration' => [
          'output_format' => 'U',
        ],
        'value' => 42,
      ],
    ];
  }

  /**
   * @phpstan-param array<string, mixed> $expected
   * @phpstan-param array<string, mixed> $configuration
   */
  #[DataProvider('casesTransform')]
  public function testTransform(array $expected, array $configuration, mixed $value): void {
    $pluginId = 'migrate_dc_str_to_time';
    $pluginDefinition = [];

    $container = \Drupal::getContainer();

    $row = new MigrateRow();
    $destinationProperty = '';

    /** @var \Drupal\migrate\Plugin\MigrationPluginManagerInterface $migrationPluginManager */
    $migrationPluginManager = $container->get('plugin.manager.migration');
    $migration = $migrationPluginManager->createStubMigration([]);

    /** @var \Drupal\migrate\MigrateExecutableInterface $migrateExecutable */
    $migrateExecutable = new MigrateExecutable(
      $migration,
      $this,
    );

    $expected += [
      'currentTime' => $expected['requestTime'],
    ];

    $container = \Drupal::getContainer();
    $container->get('datetime.time');
    $container->get('date.formatter');

    $processor = MigrateDcStrToTime::create(
      $this->container,
      $configuration,
      $pluginId,
      $pluginDefinition,
    );

    static::assertSame(
      $expected['value'],
      $processor->transform(
        $value,
        $migrateExecutable,
        $row,
        $destinationProperty,
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function display($message, $type = 'status'): void {
    $this->assertTrue($type == 'status', $message);
  }

}
