<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Unit\Plugin\migrate\process;

use Drupal\migrate\MigrateException;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\Row as MigrateRow;
use Drupal\migrate_dc\Plugin\migrate\process\MigrateDcFileContent;
use Drupal\Tests\UnitTestCase;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;

#[Group('migrate_dc')]
#[CoversClass(MigrateDcFileContent::class)]
class MigrateDcFileContentTest extends UnitTestCase {

  /**
   * @return array<string, array<string, mixed>>
   */
  public static function casesTransformSuccess(): array {
    $fileContent = 'my file content';
    $source = base64_encode($fileContent);

    return [
      'file exists' => [
        'expected' => $fileContent,
        'configuration' => [
          'path' => 'data:/',
          'file_name' => "text/plain;base64,$source",
        ],
        'destination' => '',
      ],
      'file exists with @ in path' => [
        'expected' => $fileContent,
        'configuration' => [
          'path' => '@data:/',
          'file_name' => "text/plain;base64,$source",
        ],
        'destination' => 'data:/',
      ],
      'file does not exist' => [
        'expected' => '',
        'configuration' => [
          'path' => 'foo',
          'file_name' => 'bar',
        ],
        'destination' => 'data:/',
      ],
    ];
  }

  /**
   * @phpstan-param array<string, mixed> $configuration
   */
  #[DataProvider('casesTransformSuccess')]
  public function testTransformSuccess(string $expected, array $configuration, string $destination): void {
    $pluginId = '';
    $pluginDefinition = [];
    $value = '';

    /** @var \Drupal\migrate\MigrateExecutable $migrateExecutable */
    $migrateExecutable = $this->createMock(MigrateExecutable::class);
    $row = new MigrateRow();
    $row->setDestinationProperty($destination, $destination);
    $destinationProperty = '';

    $processor = new MigrateDcFileContent(
      $configuration,
      $pluginId,
      $pluginDefinition,
    );

    static::assertSame(
      $expected,
      $processor->transform(
        $value,
        $migrateExecutable,
        $row,
        $destinationProperty,
      )
    );
  }

  /**
   * @return array<string, array<string, mixed>>
   */
  public static function casesTransformFail(): array {
    return [
      'no file in config' => [
        'expected' => MigrateException::class,
        'configuration' => [],
        'destination' => '',
      ],
    ];
  }

  /**
   * @phpstan-param class-string<\Throwable> $expected
   * @phpstan-param array<string, mixed> $configuration
   */
  #[DataProvider('casesTransformFail')]
  public function testTransformFail(string $expected, array $configuration, string $destination): void {
    $pluginId = '';
    $pluginDefinition = [];
    $value = '';

    /** @var \Drupal\migrate\MigrateExecutable $migrateExecutable */
    $migrateExecutable = $this->createMock(MigrateExecutable::class);
    $row = new MigrateRow();
    $row->setDestinationProperty($destination, $destination);
    $destinationProperty = '';

    $processor = new MigrateDcFileContent(
      $configuration,
      $pluginId,
      $pluginDefinition,
    );

    $this->expectException($expected);
    $this->expectExceptionMessage('Config path value does not exist');

    $processor->transform(
      $value,
      $migrateExecutable,
      $row,
      $destinationProperty,
    );
  }

}
