<?php

declare(strict_types=1);

namespace Drupal\Tests\migrate_dc\Unit\Plugin\migrate\source;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Serialization\SerializationInterface;
use Drupal\migrate\Plugin\MigrateSourceInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate_dc\Plugin\migrate\source\MigrateDcJsonSource;
use Drupal\Tests\UnitTestCase;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[Group('migrate_dc')]
#[CoversClass(MigrateDcJsonSource::class)]
class MigrateDcJsonSourceTest extends UnitTestCase {

  /**
   * @phpstan-var class-string<\Drupal\Component\Serialization\SerializationInterface>
   */
  protected static string $serializerClass = Json::class;

  protected static ?SerializationInterface $serializer = NULL;

  protected static function getSerializer(): SerializationInterface {
    if (static::$serializer === NULL) {
      static::$serializer = new static::$serializerClass();
    }

    return static::$serializer;
  }

  /**
   * @return array<string, array<string, mixed>>
   */
  public static function casesConstructorSuccess(): array {
    $serializer = static::getSerializer();

    $source = base64_encode($serializer->encode([
      'id-01' => [
        'a' => 'A',
        'b' => 'B',
      ],
    ]));

    return [
      'basic' => [
        'expected' => [
          'items' => [
            'id-01' => [
              'a' => 'A',
              'b' => 'B',
              'id' => 'id-01',
            ],
          ],
        ],
        'configuration' => [
          'fileDir' => 'data:/',
          'fileName' => "text/plain;base64,$source",
          'fields' => [
            'a' => 'Field A',
            'b' => 'Field B',
            'id' => 'Field ID',
          ],
          'identifier' => 'id',
        ],
      ],
      'identifier array' => [
        'expected' => [
          'items' => [
            'id-01' => [
              'a' => 'A',
              'b' => 'B',
              'id' => 'id-01',
            ],
          ],
        ],
        'configuration' => [
          'fileDir' => 'data:/',
          'fileName' => "text/plain;base64,$source",
          'fields' => [
            'a' => 'Field A',
            'b' => 'Field B',
            'id' => 'Field ID',
          ],
          'identifier' => [
            'id' => [
              'type' => 'string',
            ],
          ],
        ],
      ],
    ];
  }

  /**
   * @param array<string, mixed> $expected
   * @param array<string, mixed> $configuration
   */
  #[DataProvider('casesConstructorSuccess')]
  public function testConstructorSuccess(array $expected, array $configuration): void {
    $source = $this->createInstance($configuration);

    static::assertSameSize($expected['items'], $source);

    while ($source->valid()) {
      $id = $source->key();
      $item = $source->current();

      static::assertSame($expected['items'][$id], $item);
      $source->next();
    }

    static::assertSame($configuration['fields'], $source->fields());
  }

  /**
   * @param array<string, mixed> $configuration
   */
  protected function createInstance(array $configuration): MigrateSourceInterface {
    $pluginId = 'migrate_dc_json';
    $pluginDefinition = [];

    /** @var \Drupal\migrate\Plugin\MigrationInterface|\PHPUnit\Framework\MockObject\MockObject $migration */
    $migration = $this->createMock(MigrationInterface::class);

    /** @var \Symfony\Component\DependencyInjection\ContainerInterface|\PHPUnit\Framework\MockObject\MockObject $container */
    $container = $this->createMock(ContainerInterface::class);
    $container
      ->method('get')
      ->with('serialization.json')
      ->willReturn($this->getSerializer());

    return MigrateDcJsonSource::create(
      $container,
      $configuration,
      $pluginId,
      $pluginDefinition,
      $migration
    );
  }

}
